@@ -10,6 +10,15 @@ import shlex
 import subprocess
 import re
 
+if sys.platform == 'win32':
+    def clean_environ(e):
+        ret = dict(
+            ((str(k),str(v)) for k,v in e.iteritems()) )
+        return ret
+else:
+    def clean_environ(e): 
+        return e
+
 # From pathutils by Michael Foord: http://www.voidspace.org.uk/python/pathutils.html
 def onerror(func, path, exc_info):
     """
@@ -33,6 +42,53 @@ def onerror(func, path, exc_info):
 
 __all__ = ['TestFileEnvironment']
 
+if sys.platform == 'win32':
+    def full_executable_path(invoked, environ):
+
+        if os.path.splitext(invoked)[1]:
+            return invoked
+        
+        explicit_dir = os.path.dirname(invoked)
+
+        if explicit_dir:
+            path = [ explicit_dir ]
+        else:
+            path = environ.get('PATH').split(os.path.pathsep)
+
+        extensions = environ.get(
+            'PATHEXT',
+            # Use *something* in case the environment variable is
+            # empty.  These come from my machine's defaults
+            '.COM;.EXE;.BAT;.CMD;.VBS;.VBE;.JS;.JSE;.WSF;.WSH;.PSC1'
+            ).split(os.path.pathsep)
+
+        for dir in path:
+            for ext in extensions:
+                full_path = os.path.join(dir, invoked+ext)
+                if os.path.exists( full_path ):
+                    return full_path
+
+        return invoked # Not found; invoking it will likely fail
+
+    class Popen(subprocess.Popen):
+        def __init__(
+            self, args, bufsize=0, executable=None,
+            stdin=None, stdout=None, stderr=None,
+            preexec_fn=None, close_fds=False, shell=False, 
+            cwd=None, env=None, 
+            *args_, **kw):
+
+            if executable is None and not shell:
+                executable = full_executable_path(args[0], env or os.environ)
+
+            super(Popen,self).__init__(
+                args, bufsize, executable, stdin, stdout, stderr, 
+                preexec_fn, close_fds, shell, cwd, env, *args_, **kw)
+        
+else:
+    from subprocess import Popen
+
+
 class TestFileEnvironment(object):
 
     """
@@ -46,7 +102,7 @@ class TestFileEnvironment(object):
     def __init__(self, base_path=None, template_path=None,
                  environ=None, cwd=None, start_clear=True,
                  ignore_paths=None, ignore_hidden=True,
-                 capture_temp=False, assert_no_temp=False):
+                 capture_temp=False, assert_no_temp=False, split_cmd=True):
         """
         Creates an environment.  ``base_path`` is used as the current
         working directory, and generally where changes are looked for.
@@ -97,11 +153,15 @@ class TestFileEnvironment(object):
             os.makedirs(base_path)
         self.ignore_paths = ignore_paths or []
         self.ignore_hidden = ignore_hidden
+        self.split_cmd = split_cmd
+
         if assert_no_temp and not self.capture_temp:
             raise TypeError(
                 'You cannot use assert_no_temp unless capture_temp=True')
         self._assert_no_temp = assert_no_temp
 
+        self.split_cmd = split_cmd
+
     def _guess_base_path(self, stack_level):
         frame = sys._getframe(stack_level+1)
         file = frame.f_globals.get('__file__')
@@ -140,6 +200,7 @@ class TestFileEnvironment(object):
         cwd = _popget(kw, 'cwd', self.cwd)
         stdin = _popget(kw, 'stdin', None)
         quiet = _popget(kw, 'quiet', False)
+        debug = _popget(kw, 'debug', False)
         if not self.temp_path:
             if 'expect_temp' in kw:
                 raise TypeError(
@@ -148,26 +209,40 @@ class TestFileEnvironment(object):
         args = map(str, args)
         assert not kw, (
             "Arguments not expected: %s" % ', '.join(kw.keys()))
-        if ' ' in script:
+        if self.split_cmd and ' ' in script:
             if args:
                 # Then treat this as a script that has a space in it
                 pass
             else:
                 script, args = script.split(None, 1)
                 args = shlex.split(args)
-        # We don't want to resolve $PATH for this:
-        all_proc_results = [script] + args
+        
+        environ=clean_environ(self.environ)
         all = [script] + args
+
         files_before = self._find_files()
-        proc = subprocess.Popen(all, stdin=subprocess.PIPE,
-                                stderr=subprocess.PIPE,
-                                stdout=subprocess.PIPE,
-                                cwd=cwd,
-                                env=self.environ)
-        stdout, stderr = proc.communicate(stdin)
+
+        if debug:
+            proc = subprocess.Popen(all,
+                                    cwd=cwd,
+                                    shell=(sys.platform=='win32'), # see http://bugs.python.org/issue8557
+                                    env=clean_environ(self.environ))
+        else:
+            proc = subprocess.Popen(all, stdin=subprocess.PIPE, 
+                                    stderr=subprocess.PIPE, 
+                                    stdout=subprocess.PIPE,
+                                    cwd=cwd,
+                                    shell=(sys.platform=='win32'), # see http://bugs.python.org/issue8557
+                                    env=clean_environ(self.environ))
+
+        if debug:
+            stdout,stderr = proc.communicate()
+        else:
+            stdout, stderr = proc.communicate(stdin)
+
         files_after = self._find_files()
         result = ProcResult(
-            self, all_proc_results, stdin, stdout, stderr,
+            self, all, stdin, stdout, stderr,
             returncode=proc.returncode,
             files_before=files_before,
             files_after=files_after)
